@@ -1,4 +1,6 @@
 # coding: utf-8
+from __future__ import unicode_literals
+
 import os
 import sys
 
@@ -70,6 +72,7 @@ class ConfigTest(TestCase):
                 'verify': True,
                 'cert': None,
                 'name': None,
+                'retry': 0,
             })
 
         with env(CLOUDSTACK_KEY='test key from env',
@@ -77,6 +80,7 @@ class ConfigTest(TestCase):
                  CLOUDSTACK_ENDPOINT='https://api.example.com/from-env',
                  CLOUDSTACK_METHOD='post',
                  CLOUDSTACK_TIMEOUT='99',
+                 CLOUDSTACK_RETRY='5',
                  CLOUDSTACK_VERIFY='/path/to/ca.pem',
                  CLOUDSTACK_CERT='/path/to/cert.pem'):
             conf = read_config()
@@ -89,6 +93,7 @@ class ConfigTest(TestCase):
                 'verify': '/path/to/ca.pem',
                 'cert': '/path/to/cert.pem',
                 'name': None,
+                'retry': '5',
             })
 
     def test_current_dir_config(self):
@@ -97,6 +102,8 @@ class ConfigTest(TestCase):
                     'endpoint = https://api.example.com/from-file\n'
                     'key = test key from file\n'
                     'secret = test secret from file\n'
+                    'theme = monokai\n'
+                    'other = please ignore me\n'
                     'timeout = 50')
             self.addCleanup(partial(os.remove, '/tmp/cloudstack.ini'))
 
@@ -106,6 +113,7 @@ class ConfigTest(TestCase):
                 'endpoint': 'https://api.example.com/from-file',
                 'key': 'test key from file',
                 'secret': 'test secret from file',
+                'theme': 'monokai',
                 'timeout': '50',
                 'name': 'cloudstack',
             })
@@ -120,10 +128,15 @@ class RequestTest(TestCase):
         get.return_value.json.return_value = {
             'listvirtualmachinesresponse': {},
         }
-        machines = cs.listVirtualMachines(listall='true')
+        machines = cs.listVirtualMachines(listall='true',
+                                          headers={'Accept-Encoding': 'br'})
         self.assertEqual(machines, {})
         get.assert_called_once_with(
-            'localhost', timeout=20, verify=True, cert=None, params={
+            'localhost', timeout=20, verify=True, cert=None,
+            headers={
+                'Accept-Encoding': 'br',
+            },
+            params={
                 'apiKey': 'foo',
                 'response': 'json',
                 'command': 'listVirtualMachines',
@@ -141,17 +154,21 @@ class RequestTest(TestCase):
             'listvirtualmachinesresponse': {},
         }
         machines = cs.listVirtualMachines(zoneId=2, templateId='3',
-                                          temPlateidd='4')
-        self.assertEqual(machines, {})
+                                          temPlateidd='4', pageSize='10',
+                                          fetch_list=True)
+        self.assertEqual(machines, [])
         get.assert_called_once_with(
-            'localhost', timeout=20, verify=True, cert=None, params={
+            'localhost', timeout=20, verify=True, cert=None, headers=None,
+            params={
                 'apiKey': 'foo',
                 'response': 'json',
                 'command': 'listVirtualMachines',
-                'signature': 'dMRxAZcs2OPK15WUulzUtnrLWD0=',
+                'signature': 'mMS7XALuGkCXk7kj5SywySku0Z0=',
                 'templateId': '3',
                 'temPlateidd': '4',
-                'zoneId': '2'
+                'zoneId': '2',
+                'page': '1',
+                'pageSize': '10',
             },
         )
 
@@ -164,7 +181,8 @@ class RequestTest(TestCase):
         }
         cs.listVirtualMachines(listall=1, unicode_param=u'éèààû')
         get.assert_called_once_with(
-            'localhost', timeout=10, verify=True, cert=None, params={
+            'localhost', timeout=10, verify=True, cert=None, headers=None,
+            params={
                 'apiKey': 'foo',
                 'response': 'json',
                 'command': 'listVirtualMachines',
@@ -175,23 +193,68 @@ class RequestTest(TestCase):
         )
 
     @patch("requests.get")
-    def test_transformt(self, get):
+    def test_transform(self, get):
         cs = CloudStack(endpoint='localhost', key='foo', secret='bar')
         get.return_value.status_code = 200
         get.return_value.json.return_value = {
             'listvirtualmachinesresponse': {},
         }
         cs.listVirtualMachines(foo=["foo", "bar"],
-                               bar=[{'baz': 'blah', 'foo': 'meh'}])
+                               bar=[{'baz': 'blah', 'foo': 1000}],
+                               bytes_param=b'blah')
         get.assert_called_once_with(
-            'localhost', timeout=10, cert=None, verify=True, params={
+            'localhost', timeout=10, cert=None, verify=True, headers=None,
+            params={
                 'command': 'listVirtualMachines',
                 'response': 'json',
-                'bar[0].foo': 'meh',
+                'bar[0].foo': '1000',
                 'bar[0].baz': 'blah',
                 'foo': 'foo,bar',
+                'bytes_param': b'blah',
                 'apiKey': 'foo',
-                'signature': 'UGUVEfCOfGfOlqoTj1D2m5adr2g=',
+                'signature': 'ImJ/5F0P2RDL7yn4LdLnGcEx5WE=',
+            },
+        )
+
+    @patch("requests.get")
+    def test_transform_dict(self, get):
+        cs = CloudStack(endpoint='localhost', key='foo', secret='bar')
+        get.return_value.status_code = 200
+        get.return_value.json.return_value = {
+            'scalevirtualmachineresponse': {},
+        }
+        cs.scaleVirtualMachine(id='a',
+                               details={'cpunumber': 1000, 'memory': '640k'})
+        get.assert_called_once_with(
+            'localhost', timeout=10, cert=None, verify=True, headers=None,
+            params={
+                'command': 'scaleVirtualMachine',
+                'response': 'json',
+                'id': 'a',
+                'details[0].cpunumber': '1000',
+                'details[0].memory': '640k',
+                'apiKey': 'foo',
+                'signature': 'ZNl66z3gFhnsx2Eo3vvCIM0kAgI=',
+            },
+        )
+
+    @patch("requests.get")
+    def test_transform_empty(self, get):
+        cs = CloudStack(endpoint='localhost', key='foo', secret='bar')
+        get.return_value.status_code = 200
+        get.return_value.json.return_value = {
+            'createnetworkresponse': {},
+        }
+        cs.createNetwork(name="", display_text="")
+        get.assert_called_once_with(
+            'localhost', timeout=10, cert=None, verify=True, headers=None,
+            params={
+                'command': 'createNetwork',
+                'response': 'json',
+                'name': '',
+                'display_text': '',
+                'apiKey': 'foo',
+                'signature': 'CistTEiPt/4Rv1v4qSyILvPbhmg=',
             },
         )
 
@@ -207,13 +270,16 @@ class RequestTest(TestCase):
         cs.listVirtualMachines(blah='brah')
         self.assertEqual(get.call_args_list, [])
         self.assertEqual(post.call_args_list, [
-            call('localhost', timeout=10, verify=True, cert=None, data={
-                'command': 'listVirtualMachines',
-                'blah': 'brah',
-                'apiKey': 'foo',
-                'response': 'json',
-                'signature': '58VvLSaVUqHnG9DhXNOAiDFwBoA=',
-            })]
+            call(
+                'localhost', timeout=10, verify=True, cert=None, headers=None,
+                data={
+                   'command': 'listVirtualMachines',
+                   'blah': 'brah',
+                   'apiKey': 'foo',
+                   'response': 'json',
+                   'signature': '58VvLSaVUqHnG9DhXNOAiDFwBoA=',
+                }
+            )]
         )
 
     @patch("requests.get")
